@@ -1,18 +1,20 @@
 """ miscellaneous sorting / groupby utilities """
-
 import numpy as np
-from pandas.compat import long, string_types, PY3
-from pandas.core.dtypes.common import (
-    _ensure_platform_int,
-    _ensure_int64,
-    is_list_like,
-    is_categorical_dtype)
-from pandas.core.dtypes.cast import infer_dtype_from_array
-from pandas.core.dtypes.missing import isna
-import pandas.core.algorithms as algorithms
-from pandas._libs import lib, algos, hashtable
+
+from pandas._libs import algos, hashtable, lib
 from pandas._libs.hashtable import unique_label_indices
 
+from pandas.core.dtypes.cast import infer_dtype_from_array
+from pandas.core.dtypes.common import (
+    ensure_int64,
+    ensure_platform_int,
+    is_categorical_dtype,
+    is_extension_array_dtype,
+    is_list_like,
+)
+from pandas.core.dtypes.missing import isna
+
+import pandas.core.algorithms as algorithms
 
 _INT64_MAX = np.iinfo(np.int64).max
 
@@ -44,21 +46,36 @@ def get_group_index(labels, shape, sort, xnull):
     An array of type int64 where two elements are equal if their corresponding
     labels are equal at all location.
     """
+
     def _int64_cut_off(shape):
-        acc = long(1)
+        acc = 1
         for i, mul in enumerate(shape):
-            acc *= long(mul)
+            acc *= int(mul)
             if not acc < _INT64_MAX:
                 return i
         return len(shape)
 
-    def loop(labels, shape):
+    def maybe_lift(lab, size):
+        # promote nan values (assigned -1 label in lab array)
+        # so that all output values are non-negative
+        return (lab + 1, size + 1) if (lab == -1).any() else (lab, size)
+
+    labels = map(ensure_int64, labels)
+    if not xnull:
+        labels, shape = map(list, zip(*map(maybe_lift, labels, shape)))
+
+    labels = list(labels)
+    shape = list(shape)
+
+    # Iteratively process all the labels in chunks sized so less
+    # than _INT64_MAX unique int ids will be required for each chunk
+    while True:
         # how many levels can be done without overflow:
         nlev = _int64_cut_off(shape)
 
         # compute flat ids for the first `nlev` levels
-        stride = np.prod(shape[1:nlev], dtype='i8')
-        out = stride * labels[0].astype('i8', subok=False, copy=False)
+        stride = np.prod(shape[1:nlev], dtype="i8")
+        out = stride * labels[0].astype("i8", subok=False, copy=False)
 
         for i in range(1, nlev):
             if shape[i] == 0:
@@ -74,7 +91,7 @@ def get_group_index(labels, shape, sort, xnull):
             out[mask] = -1
 
         if nlev == len(shape):  # all levels done!
-            return out
+            break
 
         # compress what has been done so far in order to avoid overflow
         # to retain lexical ranks, obs_ids should be sorted
@@ -83,16 +100,7 @@ def get_group_index(labels, shape, sort, xnull):
         labels = [comp_ids] + labels[nlev:]
         shape = [len(obs_ids)] + shape[nlev:]
 
-        return loop(labels, shape)
-
-    def maybe_lift(lab, size):  # pormote nan values
-        return (lab + 1, size + 1) if (lab == -1).any() else (lab, size)
-
-    labels = map(_ensure_int64, labels)
-    if not xnull:
-        labels, shape = map(list, zip(*map(maybe_lift, labels, shape)))
-
-    return loop(list(labels), list(shape))
+    return out
 
 
 def get_compressed_ids(labels, sizes):
@@ -117,9 +125,9 @@ def get_compressed_ids(labels, sizes):
 
 
 def is_int64_overflow_possible(shape):
-    the_prod = long(1)
+    the_prod = 1
     for x in shape:
-        the_prod *= long(x)
+        the_prod *= int(x)
 
     return the_prod >= _INT64_MAX
 
@@ -129,7 +137,7 @@ def decons_group_index(comp_labels, shape):
     if is_int64_overflow_possible(shape):
         # at some point group indices are factorized,
         # and may not be deconstructed here! wrong path!
-        raise ValueError('cannot deconstruct factorized group indices!')
+        raise ValueError("cannot deconstruct factorized group indices!")
 
     label_list = []
     factor = 1
@@ -155,17 +163,16 @@ def decons_obs_group_ids(comp_ids, obs_ids, shape, labels, xnull):
     """
 
     if not xnull:
-        lift = np.fromiter(((a == -1).any() for a in labels), dtype='i8')
-        shape = np.asarray(shape, dtype='i8') + lift
+        lift = np.fromiter(((a == -1).any() for a in labels), dtype="i8")
+        shape = np.asarray(shape, dtype="i8") + lift
 
     if not is_int64_overflow_possible(shape):
         # obs ids are deconstructable! take the fast route!
         out = decons_group_index(obs_ids, shape)
-        return out if xnull or not lift.any() \
-            else [x - y for x, y in zip(out, lift)]
+        return out if xnull or not lift.any() else [x - y for x, y in zip(out, lift)]
 
     i = unique_label_indices(comp_ids)
-    i8copy = lambda a: a.astype('i8', subok=False, copy=True)
+    i8copy = lambda a: a.astype("i8", subok=False, copy=True)
     return [i8copy(lab[i]) for lab in labels]
 
 
@@ -181,8 +188,8 @@ def indexer_from_factorized(labels, shape, compress=True):
     return get_group_index_sorter(ids, ngroups)
 
 
-def lexsort_indexer(keys, orders=None, na_position='last'):
-    from pandas.core.categorical import Categorical
+def lexsort_indexer(keys, orders=None, na_position="last"):
+    from pandas.core.arrays import Categorical
 
     labels = []
     shape = []
@@ -201,22 +208,22 @@ def lexsort_indexer(keys, orders=None, na_position='last'):
         else:
             c = Categorical(key, ordered=True)
 
-        if na_position not in ['last', 'first']:
-            raise ValueError('invalid na_position: {!r}'.format(na_position))
+        if na_position not in ["last", "first"]:
+            raise ValueError("invalid na_position: {!r}".format(na_position))
 
         n = len(c.categories)
         codes = c.codes.copy()
 
-        mask = (c.codes == -1)
+        mask = c.codes == -1
         if order:  # ascending
-            if na_position == 'last':
+            if na_position == "last":
                 codes = np.where(mask, n, codes)
-            elif na_position == 'first':
+            elif na_position == "first":
                 codes += 1
         else:  # not order means descending
-            if na_position == 'last':
+            if na_position == "last":
                 codes = np.where(mask, n, n - codes - 1)
-            elif na_position == 'first':
+            elif na_position == "first":
                 codes = np.where(mask, 0, n - codes)
         if mask.any():
             n += 1
@@ -227,20 +234,23 @@ def lexsort_indexer(keys, orders=None, na_position='last'):
     return indexer_from_factorized(labels, shape)
 
 
-def nargsort(items, kind='quicksort', ascending=True, na_position='last'):
+def nargsort(items, kind="quicksort", ascending=True, na_position="last"):
     """
     This is intended to be a drop-in replacement for np.argsort which
     handles NaNs. It adds ascending and na_position parameters.
     GH #6399, #5231
     """
+    from pandas.core.internals.arrays import extract_array
 
-    # specially handle Categorical
-    if is_categorical_dtype(items):
-        return items.argsort(ascending=ascending, kind=kind)
+    items = extract_array(items)
+    mask = np.asarray(isna(items))
+
+    if is_extension_array_dtype(items):
+        items = items._values_for_argsort()
+    else:
+        items = np.asanyarray(items)
 
-    items = np.asanyarray(items)
     idx = np.arange(len(items))
-    mask = isna(items)
     non_nans = items[~mask]
     non_nan_idx = idx[~mask]
     nan_idx = np.nonzero(mask)[0]
@@ -252,16 +262,16 @@ def nargsort(items, kind='quicksort', ascending=True, na_position='last'):
         indexer = indexer[::-1]
     # Finally, place the NaNs at the end or the beginning according to
     # na_position
-    if na_position == 'last':
+    if na_position == "last":
         indexer = np.concatenate([indexer, nan_idx])
-    elif na_position == 'first':
+    elif na_position == "first":
         indexer = np.concatenate([nan_idx, indexer])
     else:
-        raise ValueError('invalid na_position: {!r}'.format(na_position))
+        raise ValueError("invalid na_position: {!r}".format(na_position))
     return indexer
 
 
-class _KeyMapper(object):
+class _KeyMapper:
 
     """
     Ease my suffering. Map compressed group id -> key tuple
@@ -273,8 +283,7 @@ class _KeyMapper(object):
         self.comp_ids = comp_ids.astype(np.int64)
 
         self.k = len(labels)
-        self.tables = [hashtable.Int64HashTable(ngroups)
-                       for _ in range(self.k)]
+        self.tables = [hashtable.Int64HashTable(ngroups) for _ in range(self.k)]
 
         self._populate_tables()
 
@@ -283,8 +292,10 @@ class _KeyMapper(object):
             table.map(self.comp_ids, labs.astype(np.int64))
 
     def get_key(self, comp_id):
-        return tuple(level[table.get_item(comp_id)]
-                     for table, level in zip(self.tables, self.levels))
+        return tuple(
+            level[table.get_item(comp_id)]
+            for table, level in zip(self.tables, self.levels)
+        )
 
 
 def get_flattened_iterator(comp_ids, ngroups, levels, labels):
@@ -298,9 +309,11 @@ def get_indexer_dict(label_list, keys):
     shape = list(map(len, keys))
 
     group_index = get_group_index(label_list, shape, sort=True, xnull=True)
-    ngroups = ((group_index.size and group_index.max()) + 1) \
-        if is_int64_overflow_possible(shape) \
-        else np.prod(shape, dtype='i8')
+    ngroups = (
+        ((group_index.size and group_index.max()) + 1)
+        if is_int64_overflow_possible(shape)
+        else np.prod(shape, dtype="i8")
+    )
 
     sorter = get_group_index_sorter(group_index, ngroups)
 
@@ -313,6 +326,7 @@ def get_indexer_dict(label_list, keys):
 # ----------------------------------------------------------------------
 # sorting levels...cleverly?
 
+
 def get_group_index_sorter(group_index, ngroups):
     """
     algos.groupsort_indexer implements `counting sort` and it is at least
@@ -330,14 +344,12 @@ def get_group_index_sorter(group_index, ngroups):
     count = len(group_index)
     alpha = 0.0  # taking complexities literally; there may be
     beta = 1.0  # some room for fine-tuning these parameters
-    do_groupsort = (count > 0 and ((alpha + beta * ngroups) <
-                                   (count * np.log(count))))
+    do_groupsort = count > 0 and ((alpha + beta * ngroups) < (count * np.log(count)))
     if do_groupsort:
-        sorter, _ = algos.groupsort_indexer(_ensure_int64(group_index),
-                                            ngroups)
-        return _ensure_platform_int(sorter)
+        sorter, _ = algos.groupsort_indexer(ensure_int64(group_index), ngroups)
+        return ensure_platform_int(sorter)
     else:
-        return group_index.argsort(kind='mergesort')
+        return group_index.argsort(kind="mergesort")
 
 
 def compress_group_index(group_index, sort=True):
@@ -350,7 +362,7 @@ def compress_group_index(group_index, sort=True):
     size_hint = min(len(group_index), hashtable._SIZE_HINT_LIMIT)
     table = hashtable.Int64HashTable(size_hint)
 
-    group_index = _ensure_int64(group_index)
+    group_index = ensure_int64(group_index)
 
     # note, group labels come out ascending (ie, 1,2,3 etc)
     comp_ids, obs_group_ids = table.get_labels_groupby(group_index)
@@ -381,7 +393,7 @@ def _reorder_by_uniques(uniques, labels):
     return uniques, labels
 
 
-def safe_sort(values, labels=None, na_sentinel=-1, assume_unique=False):
+def safe_sort(values, labels=None, na_sentinel=-1, assume_unique=False, verify=True):
     """
     Sort ``values`` and reorder corresponding ``labels``.
     ``values`` should be unique if ``labels`` is not None.
@@ -402,6 +414,12 @@ def safe_sort(values, labels=None, na_sentinel=-1, assume_unique=False):
     assume_unique : bool, default False
         When True, ``values`` are assumed to be unique, which can speed up
         the calculation. Ignored when ``labels`` is None.
+    verify : bool, default True
+        Check if labels are out of bound for the values and put out of bound
+        labels equal to na_sentinel. If ``verify=False``, it is assumed there
+        are no out of bound labels. Ignored when ``labels`` is None.
+
+        .. versionadded:: 0.25.0
 
     Returns
     -------
@@ -420,25 +438,27 @@ def safe_sort(values, labels=None, na_sentinel=-1, assume_unique=False):
         * If ``labels`` is not None and ``values`` contain duplicates.
     """
     if not is_list_like(values):
-        raise TypeError("Only list-like objects are allowed to be passed to"
-                        "safe_sort as values")
-
-    if not isinstance(values, np.ndarray):
+        raise TypeError(
+            "Only list-like objects are allowed to be passed to" "safe_sort as values"
+        )
 
+    if not isinstance(values, np.ndarray) and not is_extension_array_dtype(values):
         # don't convert to string types
         dtype, _ = infer_dtype_from_array(values)
         values = np.asarray(values, dtype=dtype)
 
     def sort_mixed(values):
         # order ints before strings, safe in py3
-        str_pos = np.array([isinstance(x, string_types) for x in values],
-                           dtype=bool)
+        str_pos = np.array([isinstance(x, str) for x in values], dtype=bool)
         nums = np.sort(values[~str_pos])
         strs = np.sort(values[str_pos])
         return np.concatenate([nums, np.asarray(strs, dtype=object)])
 
     sorter = None
-    if PY3 and lib.infer_dtype(values) == 'mixed-integer':
+    if (
+        not is_extension_array_dtype(values)
+        and lib.infer_dtype(values, skipna=False) == "mixed-integer"
+    ):
         # unorderable in py3 if mixed str/int
         ordered = sort_mixed(values)
     else:
@@ -455,31 +475,46 @@ def safe_sort(values, labels=None, na_sentinel=-1, assume_unique=False):
         return ordered
 
     if not is_list_like(labels):
-        raise TypeError("Only list-like objects or None are allowed to be"
-                        "passed to safe_sort as labels")
-    labels = _ensure_platform_int(np.asarray(labels))
+        raise TypeError(
+            "Only list-like objects or None are allowed to be"
+            "passed to safe_sort as labels"
+        )
+    labels = ensure_platform_int(np.asarray(labels))
 
     from pandas import Index
+
     if not assume_unique and not Index(values).is_unique:
         raise ValueError("values should be unique if labels is not None")
 
     if sorter is None:
         # mixed types
         (hash_klass, _), values = algorithms._get_data_algo(
-            values, algorithms._hashtables)
+            values, algorithms._hashtables
+        )
         t = hash_klass(len(values))
         t.map_locations(values)
-        sorter = _ensure_platform_int(t.lookup(ordered))
+        sorter = ensure_platform_int(t.lookup(ordered))
 
-    reverse_indexer = np.empty(len(sorter), dtype=np.int_)
-    reverse_indexer.put(sorter, np.arange(len(sorter)))
+    if na_sentinel == -1:
+        # take_1d is faster, but only works for na_sentinels of -1
+        order2 = sorter.argsort()
+        new_labels = algorithms.take_1d(order2, labels, fill_value=-1)
+        if verify:
+            mask = (labels < -len(values)) | (labels >= len(values))
+        else:
+            mask = None
+    else:
+        reverse_indexer = np.empty(len(sorter), dtype=np.int_)
+        reverse_indexer.put(sorter, np.arange(len(sorter)))
+        # Out of bound indices will be masked with `na_sentinel` next, so we
+        # may deal with them here without performance loss using `mode='wrap'`
+        new_labels = reverse_indexer.take(labels, mode="wrap")
 
-    mask = (labels < -len(values)) | (labels >= len(values)) | \
-        (labels == na_sentinel)
+        mask = labels == na_sentinel
+        if verify:
+            mask = mask | (labels < -len(values)) | (labels >= len(values))
 
-    # (Out of bound indices will be masked with `na_sentinel` next, so we may
-    # deal with them here without performance loss using `mode='wrap'`.)
-    new_labels = reverse_indexer.take(labels, mode='wrap')
-    np.putmask(new_labels, mask, na_sentinel)
+    if mask is not None:
+        np.putmask(new_labels, mask, na_sentinel)
 
-    return ordered, _ensure_platform_int(new_labels)
+    return ordered, ensure_platform_int(new_labels)
